@@ -32,6 +32,7 @@ GNU General Public License for more details.
 
 #include <openbabel/base.h>
 #include <openbabel/residue.h>
+#include <openbabel/math/vector3.h>
 
 namespace OpenBabel
 {
@@ -69,9 +70,10 @@ namespace OpenBabel
 #define OB_POS_CHIRAL_ATOM (1<<10)
   //! Atom has - chiral volume
 #define OB_NEG_CHIRAL_ATOM (1<<11)
-  //! Atom has no hydrogen attached. Temporary use only during SMILES input
+  //! Atom has no hydrogen attached. Temporary use only during input of some formats
 #define OB_ATOM_HAS_NO_H   (1<<12)
-  // 13-16 currently unused
+  //! Atom is not hydrogen deficient. (for SMILES input)
+#define OB_ATOM_NOT_H_DEFICIENT (1<<13)
 
   // Class OBAtom
   // class introduction in atom.cpp
@@ -114,6 +116,9 @@ namespace OpenBabel
       virtual ~OBAtom();
       //! Assignment
       OBAtom &operator = (OBAtom &);
+      //! Duplicate another atom. Copies all information with the exception of index
+      //! \since version 2.2
+      void Duplicate(OBAtom *);
       //! Clear all data. Calls OBBase::Clear() to handle any generic data.
       //! \return True if successful.
       bool Clear();
@@ -141,7 +146,7 @@ namespace OpenBabel
       //! Set the atomic type symbol (see OBTypeTable and OBAtomTyper for more)
       void SetType(const char *type);
       //! Set the atomic type symbol (see OBTypeTable and OBAtomTyper for more)
-      void SetType(std::string &type);
+      void SetType(const std::string &type);
       //! Set the partial charge to @p pcharge
       void SetPartialCharge(double pcharge){ _pcharge = pcharge; }
       //! Set the coordinate vector for this atom to @p v as a vector3
@@ -193,7 +198,7 @@ namespace OpenBabel
       unsigned int GetAtomicNum()     const { return((unsigned int)_ele); }
       //! \return the isotope for this atom, if specified, or 0 for unspecified
       unsigned short int GetIsotope() const { return(_isotope);    }
-      //! \return the atomic spin, e.g., 0 (default) for singlet,
+      //! \return the atomic spin, e.g., 0 (default) for normal atoms - note that this value is a convention,
       //!   2 for radical  1 or 3 for carbene
       int          GetSpinMultiplicity() const { return(_spinmultiplicity); }
       //! \return the atomic mass of this atom given by standard IUPAC
@@ -226,26 +231,26 @@ namespace OpenBabel
       char        *GetType();
 
       //! \return the x coordinate
-      double      GetX()    {        return(x());    }
+      double      GetX() const   {        return(x());    }
       //! \return the y coordinate
-      double      GetY()    {        return(y());    }
+      double      GetY() const  {        return(y());    }
       //! \return the z coordinate
-      double      GetZ()    {        return(z());    }
+      double      GetZ() const  {        return(z());    }
 
       // These methods check to see if there is a coordinate pointer
       // or an internal vector (e.g., SetCoordPtr())
       //! \return the x coordinate
-      double      x() {
+      double      x() const {
         if (_c)            return((*_c)[_cidx]);
         else               return _v.x();
       }
       //! \return the y coordinate
-      double      y() {
+      double      y() const {
         if (_c)            return((*_c)[_cidx+1]);
         else               return _v.y();
       }
       //! \return the z coordinate
-      double      z() {
+      double      z() const {
         if (_c)            return((*_c)[_cidx+2]);
         else               return _v.z();
       }
@@ -259,11 +264,15 @@ namespace OpenBabel
       }
       //! \return the coordinates as a vector3 object
       vector3   &GetVector();
+      //! \return the coordinates as a vector3 object
       const vector3   &GetVector() const;
       //! \return the partial charge of this atom, calculating a Gasteiger charge if needed
       double     GetPartialCharge();
       //! \return the residue which contains this atom, or NULL if none exists
       OBResidue *GetResidue();
+      //! \param perception implies whether chain perception should occur
+      //! \return the residue which contains this atom, or NULL if none exists
+      OBResidue *GetResidue(bool perception);
       //! \return the molecule which contains this atom, or NULL if none exists
       OBMol     *GetParent()        {return((OBMol*)_parent);}
       //! Create a vector for a new bond from this atom, with length given by the supplied parameter
@@ -355,6 +364,13 @@ namespace OpenBabel
       void ForceNoH() {SetFlag(OB_ATOM_HAS_NO_H);}
       //! \return if atom has been marked as having no hydrogens attached
       bool HasNoHForced() {return HasFlag(OB_ATOM_HAS_NO_H);}
+
+      //! Mark that atom is not hydrogen deficient (For SMILES input)
+      //! \since version 2.2
+      void ForceImplH() {SetFlag(OB_ATOM_NOT_H_DEFICIENT);}
+      //! \return if atom has been marked as having no hydrogens attached
+      //! \since version 2.2
+      bool HasImplHForced() {return HasFlag(OB_ATOM_NOT_H_DEFICIENT);}
       //@}
 
       //! \name Property information
