@@ -25,7 +25,7 @@ GNU General Public License for more details.
 #include <vector>
 #include <string>
 
-#include <openbabel/pluginiter.h>
+#include <openbabel/plugin.h>
 
 #ifndef OBFPRT
 #define OBFPRT
@@ -36,35 +36,45 @@ namespace OpenBabel
   class OBBase; //Forward declaration; used only as pointer.
 
 /// \brief The base class for fingerprints
-class OBFPRT OBFingerprint
+class OBFPRT OBFingerprint : public OBPlugin
 {
 //see end of cpp file for detailed documentation
 
-MAKE_PLUGIN(OBFingerprint);
+MAKE_PLUGIN(OBFingerprint)
 
+const char* TypeID()
+	{
+		return "fingerprints";
+	}
+	
+	//Rest of OBFingerprints declarations
 public:
 
   virtual ~OBFingerprint(){}
 
   /// Sets the nth bit
-  void SetBit(std::vector<unsigned int>& vec, unsigned int n);	
+  void SetBit(std::vector<unsigned int>& vec, const unsigned int n);
 
-  /// Repeatedly ORs the top half with the bottom half until no smaller than nbits 
+  ///return true if the nth bit is set;
+  bool GetBit(const std::vector<unsigned int>& vec, const unsigned int n);
+
+    /// Repeatedly ORs the top half with the bottom half until no smaller than nbits 
   void Fold(std::vector<unsigned int>& vec, unsigned int nbits); 
 
   /// \return fingerprint in vector, which may be resized, folded to nbits (if nbits!=0)
   virtual bool GetFingerprint(OBBase* pOb, std::vector<unsigned int>& fp, int nbits=0)=0;
 
-  /// Required short description of the fingerprint type.
-  virtual std::string Description()=0;
-
   /// Optional flags
   enum FptFlag{FPT_UNIQUEBITS=1};
   virtual unsigned int Flags() { return 0;}; 
 
-  // Obtain info on available fingerprints
-  // Replaced by FOR_EACH(OBFingerprint)
-//  static bool GetNextFPrt(std::string& id, OBFingerprint*& pFPrt);
+  /// \return a description of each bit that is set (or unset, if bSet=false)
+  /// \since version 2.2
+  virtual std::string DescribeBits(const std::  vector<unsigned int> fp, bool bSet=true)
+  {
+    std::string txt("");
+    return txt;
+  }
 
   /// \return the Tanimoto coefficient between two vectors (vector<unsigned int>& SeekPositions)
   static double Tanimoto(const std::vector<unsigned int>& vec1, const std::vector<unsigned int>& vec2);
@@ -79,11 +89,20 @@ public:
     {
       int andfp = vec1[i] & p2[i];
       int orfp = vec1[i] | p2[i];
-      //Count bits
+      // Count bits
+      /* GCC 3.4 supports a "population count" builtin, which on many targets is
+         implemented with a single instruction.  There is a fallback definition
+         in libgcc in case a target does not have one, which should be just as
+         good as the static function below.  */
+#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
+      andbits += __builtin_popcount(andfp);
+      orbits += __builtin_popcount(orfp);
+#else
       for(;andfp;andfp=andfp<<1)
         if(andfp<0) ++andbits;
       for(;orfp;orfp=orfp<<1)
         if(orfp<0) ++orbits;
+#endif
     }
       return((double)andbits/(double)orbits);
   };
@@ -103,14 +122,13 @@ private:
 
 public:
 /// \return a pointer to a fingerprint (the default if ID is empty), or NULL if not available
-///For backward compatibility; prefer FindType (in pluginiter.h) instead
-static OBFingerprint* FindFingerprint(const std::string& ID){ return Iter().FindType(ID);}
+  ///For backward compatibility;  a synonym of OBFingerprint::FindType
+static OBFingerprint* FindFingerprint(const char* ID){ return FindType(ID);}
 
 private:
   static const unsigned int bitsperint;// = 8 * sizeof(unsigned int);
 };
 
-//*************************************************************
 //Fast search routines
 /// \struct FptIndexHeader fingerprint.h <openbabel/fingerprint.h>
 /// \brief Header for fastsearch index file
@@ -176,7 +194,6 @@ private:
   OBFingerprint* _pFP;
 };
 
-//**********************************************
 /// \class FastSearchIndexer fingerprint.h <openbabel/fingerprint.h>
 /// \brief Class to prepare fingerprint index files See FastSearch class for details
 class OBFPRT FastSearchIndexer
