@@ -29,6 +29,7 @@ GNU General Public License for more details.
 #endif
 
 #include <math.h>
+#include <float.h>
 
 #include <vector>
 #include <string>
@@ -94,6 +95,8 @@ namespace OpenBabel
 #define OB_TCHARGE_MOL		       (1<<16)
   //! Total spin on this molecule has been set. See OBMol::SetTotalSpinMultiplicity
 #define OB_TSPIN_MOL             (1<<17)
+  //! Ring typing has been performed. See OBRingTyper
+#define OB_RINGTYPES_MOL         (1<<18)
   // flags 18-32 unspecified
 #define OB_CURRENT_CONFORMER	 -1
 
@@ -108,11 +111,11 @@ namespace OpenBabel
     std::vector<OBAtom*>          _vatom;      	//!< vector of atoms
     std::vector<OBBond*>          _vbond;      	//!< vector of bonds
     unsigned short int            _dimension;   //!< Dimensionality of coordinates
-    double                        _energy;      //!< Molecular heat of formation (if applicable)
     int				  _totalCharge; //!< Total charge on the molecule
     unsigned int                  _totalSpin;   //!< Total spin on the molecule (if not specified, assumes lowest possible spin)
-    double                       *_c;	          //!< coordinate array
+    double                        *_c;	        //!< coordinate array
     std::vector<double*>          _vconf;       //!< vector of conformers
+    double                        _energy;      //!< heat of formation
     unsigned int                  _natoms;      //!< Number of atoms
     unsigned int                  _nbonds;      //!< Number of bonds
     std::vector<OBResidue*>       _residue;     //!< Residue information (if applicable)
@@ -127,7 +130,7 @@ namespace OpenBabel
     void start_kekulize(std::vector <OBAtom*> &cycle, std::vector<int> &electron);
     int expand_kekulize(OBAtom *atom1, OBAtom *atom2, std::vector<int> &currentState, std::vector<int> &initState, std::vector<int> &bcurrentState, std::vector<int> &binitState, std::vector<bool> &mark);
     int getorden(OBAtom *atom);
-    void expandcycle(OBAtom *atom, OBBitVec &avisit);
+    bool expandcycle(OBAtom *atom, OBBitVec &avisit, OBAtom *first = NULL, int depth = 0);
     //@}
 
   public:
@@ -149,7 +152,7 @@ namespace OpenBabel
     //! This improves performance since the internal atom vector does not grow.
     void ReserveAtoms(int natoms)
     {
-      if (natoms && _mod)
+      if (natoms > 0 && _mod)
         _vatom.reserve(natoms);
     }
     
@@ -196,23 +199,23 @@ namespace OpenBabel
 
     //! Create a new OBAtom in this molecule and ensure connections.
     //! (e.g. OBAtom::GetParent()
-    virtual OBAtom    *NewAtom();
+    OBAtom    *NewAtom();
     //! Create a new OBBond in this molecule and ensure connections.
     //! (e.g. OBBond::GetParent()
-    virtual OBBond    *NewBond();
+    OBBond    *NewBond();
     //! Create a new OBResidue in this molecule and ensure connections.
-    virtual OBResidue *NewResidue();
+    OBResidue *NewResidue();
     //! Deletes an atom from this molecule and all appropriate bonds.
     //! Updates the molecule and atom and bond indexes accordingly.
     //! \warning Does not update any residues which may contain this atom
     //! \return Whether deletion was successful
-    bool DeleteAtom(OBAtom*);
+    bool DeleteAtom(OBAtom*, bool destroyAtom = true);
     //! Deletes an bond from this molecule and updates accordingly
     //! \return Whether deletion was successful
-    bool DeleteBond(OBBond*);
+    bool DeleteBond(OBBond*, bool destroyBond = true);
     //! Deletes a residue from this molecule and updates accordingly.
     //! \return Whether deletion was successful
-    bool DeleteResidue(OBResidue*);
+    bool DeleteResidue(OBResidue*, bool destroyResidue = true);
     //@}
 
     //! \name Molecule modification methods
@@ -240,7 +243,8 @@ namespace OpenBabel
     //! \return the entire set of flags. (Internal use, mainly.)
     int          GetFlags()               { return(_flags); }
     //! \return the title of this molecule (often the filename)
-    const char  *GetTitle() const         { return(_title.c_str()); }
+    //! \param replaceNewlines whether to replace any newline characters with spaces
+    const char  *GetTitle(bool replaceNewlines = true) const;
     //! \return the number of atoms (i.e. OBAtom children)
     unsigned int NumAtoms() const         {  return(_natoms); }
     //! \return the number of bonds (i.e. OBBond children)
@@ -254,40 +258,46 @@ namespace OpenBabel
     
     //! \return the atom at index @p idx or NULL if it does not exist.
     //! \warning Atom indexing will change. Use iterator methods instead.
-    OBAtom      *GetAtom(int idx);
+    OBAtom      *GetAtom(int idx) const;
     //! \return the first atom in this molecule, or NULL if none exist.
     //! \deprecated Will be removed in favor of more standard iterator methods
-    OBAtom      *GetFirstAtom();
+    OBAtom      *GetFirstAtom() const;
     //! \return the bond at index @p idx or NULL if it does not exist.
     //! \warning Bond indexing may change. Use iterator methods instead.
-    OBBond      *GetBond(int idx);
+    OBBond      *GetBond(int idx) const;
     //! \return the bond connecting the atom indexed by @p a and @p b or NULL if none exists.
     //! \warning Atom indexing will change. Use atom objects and iterators instead.
-    OBBond      *GetBond(int a, int b);
+    OBBond      *GetBond(int a, int b) const;
     // The safer version of the above method
     //! \return the bond between the atoms @p bgn and @p end or NULL if none exists
-    OBBond      *GetBond(OBAtom* bgn, OBAtom* end);
+    OBBond      *GetBond(OBAtom* bgn, OBAtom* end) const;
     //! \return the residue indexed by @p idx, or NULL if none exists
     //! \warning Residue indexing may change. Use iterator methods instead.
-    OBResidue   *GetResidue(int);
+    OBResidue   *GetResidue(int idx) const;
     std::vector<OBInternalCoord*> GetInternalCoord();
-    //! \return the dihedral angle between the four atoms supplied a1-a2-a3-a4)
+    /*! \return the dihedral angle (in degrees) between the four atoms supplied a1-a2-a3-a4)
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */ 
     double       GetTorsion(int,int,int,int);
-    //! \return the dihedral angle between the four atoms @p a, @p b, @p c, and @p d)
+    /*! \return the dihedral angle (in degrees) between the four atoms @p a, @p b, @p c, and @p d)
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */ 
     double       GetTorsion(OBAtom* a,OBAtom* b,OBAtom* c,OBAtom* d);
-    //! \return the angle between the three atoms @p a, @p b and @p c
+    //! \return the angle (in degrees) between the three atoms @p a, @p b and @p c
     //!  (where  a-> b (vertex) -> c )
     double GetAngle(OBAtom* a, OBAtom* b, OBAtom* c);
     //! \return the stochoimetric formula (e.g., C4H6O)
     std::string  GetFormula();
     //! \return the stochoimetric formula in spaced format e.g. C 4 H 6 O 1
-    std::string  GetSpacedFormula(int ones=0, const char* sp=" ");
+    std::string  GetSpacedFormula(int ones=0, const char* sp=" ", bool implicitH = true);
     //! \return the heat of formation for this molecule (in kcal/mol)
-    double       GetEnergy() const { return(_energy); }
+    double       GetEnergy() const { return _energy; }
     //! \return the standard molar mass given by IUPAC atomic masses (amu)
-    double       GetMolWt();
+    double       GetMolWt(bool implicitH = true);
     //! \return the mass given by isotopes (or most abundant isotope, if not specified)
-    double	 GetExactMass();
+    double	 GetExactMass(bool implicitH = true);
     //! \return the total charge on this molecule (i.e., 0 = neutral, +1, -1...)
     int		 GetTotalCharge();
     //! \return the total spin on this molecule (i.e., 1 = singlet, 2 = doublet...)
@@ -319,9 +329,9 @@ namespace OpenBabel
     void   SetDimension(unsigned short int d) { _dimension = d; }
     //! Set the total charge of this molecule to @p charge
     void   SetTotalCharge(int charge);
-    //! Set the total spin multiplicity of this molecule to @p spin
-    //! (i.e., 0 = singlet (default), 1 = doublet, 2 = triplet, etc.)
-    void   SetTotalSpinMultiplicity(unsigned int spin);
+    //! Set the total spin multiplicity of this molecule to @p spinMultiplicity
+    //! Overrides the calculation from spin multiplicity of OBAtoms
+    void   SetTotalSpinMultiplicity(unsigned int spinMultiplicity);
     //! Set the internal coordinates to @p int_coord
     //! (Does not call InternalToCartesian to update the 3D cartesian coordinates)
     void   SetInternalCoord(std::vector<OBInternalCoord*> int_coord)
@@ -341,6 +351,8 @@ namespace OpenBabel
     void   SetRingAtomsAndBondsPerceived(){SetFlag(OB_RINGFLAGS_MOL);}
     //! Mark that atom types have been perceived (see OBAtomTyper for details)
     void   SetAtomTypesPerceived()   { SetFlag(OB_ATOMTYPES_MOL);   }
+    //! Mark that ring types have been perceived (see OBRingTyper for details)
+    void   SetRingTypesPerceived()   { SetFlag(OB_RINGTYPES_MOL);   }
     //! Mark that chains and residues have been perceived (see OBChainsParser)
     void   SetChainsPerceived()      { SetFlag(OB_CHAINS_MOL);      }
     //! Mark that chirality has been perceived
@@ -363,10 +375,13 @@ namespace OpenBabel
     void   SetFlags(int flags)       { _flags = flags;              }
 
     void   UnsetAromaticPerceived()  { _flags &= (~(OB_AROMATIC_MOL));   }
+    void   UnsetSSSRPerceived()  { _flags &= (~(OB_SSSR_MOL));   }
+    void   UnsetRingTypesPerceived()  { _flags &= (~(OB_RINGTYPES_MOL));   }
     void   UnsetPartialChargesPerceived(){ _flags &= (~(OB_PCHARGE_MOL));}
     void   UnsetImplicitValencePerceived(){_flags &= (~(OB_IMPVAL_MOL)); }
     void   UnsetHydrogensAdded()       { UnsetFlag(OB_H_ADDED_MOL);     }
     void   UnsetFlag(int flag)       { _flags &= (~(flag));              }
+    //@}
 
     //! \name Molecule modification methods
     //@{
@@ -378,6 +393,9 @@ namespace OpenBabel
     bool Clear();
     //! Renumber the atoms of this molecule according to the order in the supplied vector
     void RenumberAtoms(std::vector<OBAtom*>&);
+    //! Set the coordinates for all atoms in this conformer.
+    //! \sa OBMol::GetCoordinates()
+    void SetCoordinates(double *c);
     //! Translate one conformer and rotate by a rotation matrix (which is returned) to the inertial frame-of-reference
     void ToInertialFrame(int conf, double *rmat);
     //! Translate all conformers to the inertial frame-of-reference
@@ -417,29 +435,36 @@ namespace OpenBabel
     //! \param polaronly    Whether to add hydrogens only to polar atoms
     //! (i.e., not to C atoms)
     //! \param correctForPH Whether to call CorrectForPH() first
+    //! \param pH The pH to use for CorrectForPH() modification
     //! \return Whether any hydrogens were added
-    bool AddHydrogens(bool polaronly=false,bool correctForPH=true);
+    bool AddHydrogens(bool polaronly=false,bool correctForPH=false, double pH=7.4);
     //! Add hydrogens only to the supplied atom to fill out implicit valence
     bool AddHydrogens(OBAtom*);
     //! Add only polar hydrogens (i.e., attached to polar atoms, not C)
     bool AddPolarHydrogens();
 
     //! Deletes all atoms except for the largest contiguous fragment
-    bool StripSalts();
+    bool StripSalts(int threshold);
     //! Copies each disconnected fragment as a separate OBMol
     std::vector<OBMol> Separate(int StartIndex=1);
+    //! Iterative component of Separate to copy one fragment at a time
+    bool GetNextFragment( OpenBabel::OBMolAtomDFSIter& iter, OBMol& newMol );
     //! Converts the charged form of coordinate bonds, e.g.[N+]([O-])=O to N(=O)=O 
     bool ConvertDativeBonds();
 
     //! Correct for pH by applying the OBPhModel transformations
-    bool CorrectForPH();
+    bool CorrectForPH(double pH=7.4);
     // docs in mol.cpp
-    bool AssignSpinMultiplicity();
+    bool AssignSpinMultiplicity(bool NoImplicitH=false);
     //! \return the center of the supplied conformer @p nconf
     //! \see Center() to actually center all conformers at the origin
     vector3 Center(int nconf);
-    //! Set the torsion defined by these atoms, rotating bonded neighbors
-    void SetTorsion(OBAtom*,OBAtom*,OBAtom*,OBAtom*,double);
+    /*! Set the torsion defined by these atoms, rotating bonded neighbors
+     *  \par ang The torsion angle in radians
+     *  WARNING: SetTorsion takes an angle in radians while GetTorsion returns it
+     *  in degrees
+     */ 
+    void SetTorsion(OBAtom*,OBAtom*,OBAtom*,OBAtom*,double ang);
     //@}
 
     //! \name Molecule utilities and perception methods
@@ -496,6 +521,8 @@ namespace OpenBabel
     bool HasRingAtomsAndBondsPerceived(){return(HasFlag(OB_RINGFLAGS_MOL));}
     //! Have atom types been assigned by OBAtomTyper?
     bool HasAtomTypesPerceived()    { return(HasFlag(OB_ATOMTYPES_MOL));}
+    //! Have ring types been assigned by OBRingTyper?
+    bool HasRingTypesPerceived()    { return(HasFlag(OB_RINGTYPES_MOL));}
     //! Has atom chirality been assigned?
     bool HasChiralityPerceived()    { return(HasFlag(OB_CHIRALITY_MOL));}
     //! Have atomic Gasteiger partial charges been assigned by OBGastChrg?
@@ -542,6 +569,13 @@ namespace OpenBabel
     void    DeleteConformer(int nconf);
     //! \return the coordinates to conformer @p i
     double  *GetConformer(int i)       {  return(_vconf[i]);      }
+    //! Set the entire set of conformer energies
+    void    SetEnergies(std::vector<double> &energies);
+    //! Set the entire set of conformer energies
+    std::vector<double> GetEnergies();
+    //! Get the energy for conformer ci
+    //! \par ci conformer index
+    double  GetEnergy(int ci);
     //! Set the iterator to the beginning of the conformer list
     //! \return the array of coordinates for the first conformer
     double  *BeginConformer(std::vector<double*>::iterator&i)
@@ -561,11 +595,11 @@ namespace OpenBabel
     //! \return An atom iterator pointing to the beginning of the atom list
     OBAtomIterator BeginAtoms()   { return _vatom.begin(); }
     //! \return An atom iterator pointing to the end of the atom list
-    OBAtomIterator EndAtoms()     { return _vatom.end();   }
+     OBAtomIterator EndAtoms()    { return _vatom.begin() + NumAtoms() ; }
     //! \return A bond iterator pointing to the beginning of the bond list
     OBBondIterator BeginBonds()   { return _vbond.begin(); }
     //! \return A bond iterator pointing to the end of the bond list
-    OBBondIterator EndBonds()     { return _vbond.end();   }
+    OBBondIterator EndBonds()     { return _vbond.begin() + NumBonds() ; }
     //! \return A residue iterator pointing to the beginning of the residue list
     OBResidueIterator BeginResidues() { return _residue.begin(); }
     //! \return A residue iterator pointing to the end of the residue list
@@ -618,8 +652,9 @@ namespace OpenBabel
   };
 
   // Utility function prototypes
+  //tokenize and Trim declarations moved to base.h
   // Deprecated -- use OBMessageHandler class instead (docs in obutil.cpp)
-  OBAPI void ThrowError(const char *str);
+  OBAPI void ThrowError(char *str);
   // Deprecated -- use OBMessageHandler class instead (docs in obutil.cpp)
   OBAPI void ThrowError(std::string &str);
   //! Convert Cartesian XYZ to a set of OBInternalCoord coordinates
@@ -686,6 +721,8 @@ namespace OpenBabel
   OBAPI double superimpose(double*,double*,int);
 #endif // __KCC
 
+//  extern OBMol* (*CreateMolecule) (void);
+
 } // end namespace OpenBabel
 
 #endif // OB_MOL_H
