@@ -19,12 +19,16 @@ GNU General Public License for more details.
 #ifndef OB_MOLECULEFORMAT_H
 #define OB_MOLECULEFORMAT_H
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   #include <hash_map>
 #endif
 
+#include <typeinfo>
+
 #include <openbabel/mol.h>
+#include <openbabel/babelconfig.h>
 #include <openbabel/obconversion.h>
+#include <typeinfo>
 
 namespace OpenBabel {
 
@@ -50,6 +54,7 @@ namespace OpenBabel {
 class OBCOMMON OBMoleculeFormat : public OBFormat
 {
 private:
+  static bool OptionsRegistered;
   static std::map<std::string, OBMol*> IMols;
   static OBMol* _jmol; //!< Accumulates molecules with the -j option
   static std::vector<OBMol> MolArray; //!< Used in --separate option
@@ -59,30 +64,38 @@ public:
 
   OBMoleculeFormat()
   {
-    OBConversion::RegisterOptionParam("b", this, 0, OBConversion::INOPTIONS);
-    OBConversion::RegisterOptionParam("s", this, 0, OBConversion::INOPTIONS);
-    OBConversion::RegisterOptionParam("title", this, 1,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("addtotitle", this, 1,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("property", this, 2, OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("C",        this, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("j",        this, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("join",     this, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("separate", this, 0,OBConversion::GENOPTIONS);
+    if(!OptionsRegistered)
+    {
+      OptionsRegistered=true;
+      OBConversion::RegisterOptionParam("b", this, 0, OBConversion::INOPTIONS);
+      OBConversion::RegisterOptionParam("s", this, 0, OBConversion::INOPTIONS);
+      OBConversion::RegisterOptionParam("title", this, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("addtotitle", this, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("property", this, 2, OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("C",        this, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("j",        this, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("join",     this, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("separate", this, 0,OBConversion::GENOPTIONS);
 
-    //The follow are OBMol options, which should not be in OBConversion.
-    //But here isn't entirely appropriate either, since one could have
-    //OBMol formats loaded but which don't derived from this class.
-    //However, this possibility is remote.
-    OBConversion::RegisterOptionParam("s", NULL, 1,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("v", NULL, 1,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("h", NULL, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("d", NULL, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("b", NULL, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("c", NULL, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("p", NULL, 0,OBConversion::GENOPTIONS); 
-    OBConversion::RegisterOptionParam("t", NULL, 0,OBConversion::GENOPTIONS);
-    OBConversion::RegisterOptionParam("k", NULL, 0,OBConversion::GENOPTIONS);
-  };
+      //The follow are OBMol options, which should not be in OBConversion.
+      //But here isn't entirely appropriate either, since one could have
+      //OBMol formats loaded but which don't derived from this class.
+      //However, this possibility is remote.
+      OBConversion::RegisterOptionParam("s", NULL, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("v", NULL, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("h", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("d", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("b", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("c", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("p", NULL, 1,OBConversion::GENOPTIONS); 
+      OBConversion::RegisterOptionParam("t", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("k", NULL, 0,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("filter", NULL, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("add", NULL, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("delete", NULL, 1,OBConversion::GENOPTIONS);
+      OBConversion::RegisterOptionParam("append", NULL, 1,OBConversion::GENOPTIONS);
+    }
+  }
 
   //! Static routine,  which can be called from elsewhere
   static bool ReadChemObjectImpl(OBConversion* pConv, OBFormat*);
@@ -92,11 +105,11 @@ public:
   /// The "Convert" interface for reading a new molecule
   virtual bool ReadChemObject(OBConversion* pConv)
   { return ReadChemObjectImpl(pConv, this);}
-    
+
   /// The "Convert" interface for writing a new molecule
   virtual bool WriteChemObject(OBConversion* pConv)
   { return WriteChemObjectImpl(pConv, this);}
-  
+
   /// \name Routines to handle the -C option for combining data from several OBMols
   //@{
   //! Defer output of a molecule until later, so it can be combined with others
@@ -110,7 +123,7 @@ public:
   static OBMol* MakeCombinedMolecule(OBMol* pFirst, OBMol* pSecond);
   //@}
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   typedef stdext::hash_map<std::string, unsigned> NameIndexType;
 #else
   typedef std::map<std::string, unsigned> NameIndexType;
